--local nsg = require'nsg'
local md5 = require "md5"
local html = require 'orbiter.html'
local form = require 'orbiter.form'
local doc = require 'orbiter.doc'

local INFO = function(...)return logger:info(...)end
local WARN = function(...)return logger:warn(...)end

local USERS = {
  nsg = {
    password = ARGS.password or "nsg"
  },
}
local NONCE = {}
local hash = function(...)
  return md5.sumhexa(table.concat({...},":"))
end

local secret = hash(os.date(), "NSG")

local parseHA do
local lpeg = require'lpeg'
lpeg.locale(lpeg) 
local sp = lpeg.space^0
local C, P, S = lpeg.C, lpeg.P, lpeg.S
local name = C(lpeg.alnum^1) * sp
local quoted_string = '"' * C(((P'\\\\' + P'\\"' + P(1) - '"'))^0) * '"' * sp
local value = name + quoted_string
local sep = S(",;") * sp
local pair = lpeg.Cg(name * "=" * sp * value) * sep^-1
local list = lpeg.Cf(lpeg.Ct("") * sp * P'Digest' * sp * pair^0 * -1, rawset)
parseHA = function(str)
  return str and list:match(str)
end
end

local function isAuthorized(web)
  local ha = parseHA(web.vars.HTTP_AUTHORIZATION)
  if not ha then return false end
  local uname = ha.username or ''
  local remote = web.vars.HTTP_REMOTE
  if ha.qop and ha.qop ~= "auth" then 
    WARN("Неопознанный метод авторизации, пользователь %s@%s", uname, remote)
    return false 
  end
  local user = USERS[uname]
  if not user then
    WARN("Попытка входа незарегистрированного пользователя %s@%s", uname, remote)
    return false
  end
  local nonce = ha.nonce
  if not NONCE[nonce] then
    --WARN("Неактивная сессия, пользователь %s@%s", uname, remote)
    return false
  end
  local resp = hash(
  hash(uname, ha.realm, user.password),
  nonce,
  ha.nc,
  ha.cnonce,
  "auth",
  hash(web.method:upper(),ha.uri)
  )
  if resp == ha.response then
    web.username = uname
    web.nonce = nonce
    if NONCE[nonce] == true then
      INFO("Вход в систему управления, пользователь %s@%s(%s)", uname, remote, nonce:match("...%-..."))
      NONCE[nonce] = {user = uname}
    end
    return true
  end
  WARN("Неверный пароль, пользователь %s@%s", uname, remote)
  return false
end

local function msgUnauthorized(web)
  web.status = "401 Unauthorized"
  local time = socket.gettime()
  local nonce = ("%d"):format(time)..'-'..hash(secret, tostring(time))
  NONCE[nonce] = true
  local opaque = "5ccc069c403ebaf9f0171e9517f40e41"
  web.headers = {
      ["WWW-Authenticate"] = ([[
Digest realm="test@nsg.ru",
 qop="auth",
 nonce="%s",
 opaque="%s"
]]):format(nonce, opaque);
  }

  local msg = [[
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
 "http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd">
<HTML>
  <HEAD>
    <TITLE>Error</TITLE>
    <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=ISO-8859-1">
  </HEAD>
  <BODY><H1>401 Unauthorized.</H1></BODY>
</HTML>
]]
  return msg
end
-------------------------------------------------------------------------------
local h2,p,formt,input,div,pre = html.tags 'h2,p,form,input,div,pre'
local ul,li,input = html.tags'ul,li,input'

local function getMenuItem(web, href, name)
  local style = web.path_info == href and "font-weight:bold" or nil
  return {href, name, style = style}
end
local function menu(web)
  return html.list{
    render = html.link;
    getMenuItem(web, '/control/conf', 'Конфигурация'),
    getMenuItem(web, '/control/node', 'Узлы'),
    getMenuItem(web, '/control/sens', 'Датчики'),
    getMenuItem(web, '/control/log',  'Журнал'),
  }
end

local titleVal  = "Управление системой"
local logoutVal = "Выйти"
local delSens   = "Удалить"

local sensTreeScript = html.script[[
$(document).ready(function(){
  var color = "blue";
  $(".node").hover(
    function(){
      color = $(this).css("color");
      $(this).css("color", "#268bd2");
    },
    function(){
      $(this).css("color", color);
    }
  );
  $(".node").click(function(){
    $(this).next().toggle();
  });
});
]]
local function getSensList()
  local sensTree = {style = "list-style-type:none;";}
  for _, tunId in ipairs(TUN_LIST) do
    local tun = TUN[tunId]
    local tnam = tun.name
    local sensList = {
      style = "list-style-type:none;display:none";
    }
    for _, sensId in ipairs(tun.sensorsList) do
      snam = tun.sensors[sensId].name
      sensList[#sensList+1] = {
	input{type = "checkbox", value = tnam.."\n"..snam, name = "sensCheck"},
	snam,
      }
    end
    if #sensList > 0 then
      sensTree[#sensTree+1] = li{class = 'node';  tnam}
      sensTree[#sensTree+1] = html.list(sensList)
    end
  end

  return {
    sensTreeScript,
    formt{
      name="sens", action="/control/sens", method="post";
      input{type = "submit", value = delSens, name = "-button"},
      ul(sensTree),
    },
  }
end

local function content(web)
  local path = web.path_info
  if path == "/control/log" then
    return pre(logger:show())
  elseif path == "/control/sens" then
    return getSensList()
  end
  return p(path)
end


local control = function(self, web)
  --print("*** control ***", nsg.serialize(web,0))
  if not isAuthorized(web) then
    return msgUnauthorized(web)
  end
  if web.method == 'get' then
  else -- POST
    local bt = web.input["-button"]
    if bt == logoutVal then
      NONCE[web.nonce] = nil
      INFO("Выход из системы управления, пользователь %s@%s(%s)", 
           web.username, web.vars.HTTP_REMOTE, web.nonce:match("...%-..."))
      return html{
	favicon = "/resources/images/nsg_logo.png";
	title = titleVal;
	html.link('/control/conf','Вернуться в управление сервером'),
      }
    elseif bt == delSens then
      local sens = web.input.sensCheck
      if type(sens) ~= 'table' then
	sens = {sens}
      end
      for _, v in ipairs(sens) do
	BD:delSensor(v:match"^([^\n\r]+)[\n\r]+([^\n\r]+)$")
      end
    end
  end

  local ret = html{
    title = titleVal;
    favicon = "/resources/images/nsg_logo.png";
    styles = "/resources/css/nsgsenser.css";
    scripts = "/resources/js/jquery-1.8.3.min.js";
    div{ 
      class = "header",
      formt{
       	name="logout", action="/control/conf", method="post", style="text-align:right;";
	"Вы вошли как: "..web.username,
	input{type = "submit", value = logoutVal, name = "-button"},
      },
      h2"Управление сервером"
    },
    div{ 
      class = "layout";
      div{
	class = "sidebar";
	menu(web)
      },
      div{
	class = "content";
	content(web),
      },
    }
  }

  local head = ret:child_with_name'head'
  head[#head+1] = doc.elem('meta',{
    ['http-equiv']="Content-Type",
    content="text/html;charset=UTF-8"
  })
  return ret
end

return {
  control = control;
}
